// FaderPort 2 MIDI Device Script
// Copyright (c)2018 PreSonus Software Ltd.

// include SDK files from host
include_file ("resource://com.presonus.musicdevices/sdk/midiprotocol.js")
include_file ("resource://com.presonus.musicdevices/sdk/controlsurfacedevice.js")

//************************************************************************************************
// ColorLEDHandler
// Handler for RGB LEDs
//************************************************************************************************

ColorLEDHandler.prototype = new ControlHandler ();
function ColorLEDHandler (name, id)
{
	this.name = name;
	this.id = id;
	
	this.sendValue = function (value, flags)
	{
		let alpha = (value >> 24) & 0xFF;
		if(alpha == 0x00) // make transparent color white
			value = 0xFFFFFFFF;
		
		// 7 bit per color channel
		let r = (value >> 1) & 0x7F;
		let g = (value >> 9) & 0x7F;
		let b = (value >> 17) & 0x7F;
				
		this.sendMidi (Midi.kNoteOn|1, this.id, r);
		this.sendMidi (Midi.kNoteOn|2, this.id, g);
		this.sendMidi (Midi.kNoteOn|3, this.id, b);
	}
}

//************************************************************************************************
// FP2MidiDevice
//************************************************************************************************

FP2MidiDevice.prototype = new ControlSurfaceDevice ();
function FP2MidiDevice ()
{	
	//this.lastActiveSensing = 0;
	//this.activeSensingInterval = 1000;

	this.onInit = function (hostDevice)
	{
		ControlSurfaceDevice.prototype.onInit.call (this, hostDevice);
		
		// turn on debug output
		//this.debugLog = true;
	}
		
	this.createHandler = function (name, attributes)
	{
		// additional handlers created on the fly via <Handler> in XML
		let className = attributes.getAttribute ("class");
		if(className == "ColorLEDHandler")
		{
			let address = attributes.getAttribute ("address");
			let handler = new ColorLEDHandler (name, address);
			this.addHandler (handler);
			return true;
		}
		return false;
	}
		
	/*this.onIdle = function (time)
	{
		if(this.midiOutConnected)
		{
			if((time - this.lastActiveSensing) >= this.activeSensingInterval)
			{
				this.lastActiveSensing = time;
				this.sendMidi (Midi.kPolyPressure, 0, 0);
			}
		}		
	}*/
		
	this.onMidiOutConnected = function (state)
	{
		ControlSurfaceDevice.prototype.onMidiOutConnected.call (this, state);
		
		if(state)
		{
			// request host to retransmit all controls
			this.hostDevice.invalidateAll ();
		}
	}

	/*
	this.onSysexEvent = function (data, length)
	{
	}
	*/
}

// factory entry called by host
function createFP2DeviceInstance ()
{
	return new FP2MidiDevice;
}
